﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Serialize;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeoGis.Helper;

namespace ThinkGeoGis.GisCore
{
    [DesignTimeVisible(false), ToolboxItem(false), NonSerializableBaseType]
    [Serializable]
    public class GisPath
    {
        public PointShape pointshape1;
        public PointShape pointshape2;
        public PointShape centershape;

        public Path gisPath;

        private object SRg(params object[] param)
        {
            var result = new AssemblyHandler();
;
            var classn = result.GetClassName("ThinkGeo.MapSuite.Wpf.dll");

            Assembly assembly = Assembly.LoadFile(AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "ThinkGeo.MapSuite.Wpf.dll");
            Type type = assembly.GetType("slM=.Pxg=");
            MethodInfo medinfo = type.GetMethod("SRg=",BindingFlags.Static| BindingFlags.Public);
            object obj = medinfo.Invoke(null, param);
            return obj;
        }
        public GisPath(PointShape pt1, PointShape pt2)
        {
            gisPath = new Path();
            gisPath.Stroke = Brushes.Red;
            gisPath.StrokeThickness = 2;
            pointshape1 = pt1;
            pointshape2 = pt2;

            LineShape lineshape = new LineShape();
            lineshape.Vertices.Add(new Vertex(pointshape1));
            lineshape.Vertices.Add(new Vertex(pointshape2));
            centershape = lineshape.GetPointOnALine(StartingPoint.FirstPoint, 50f);
        }

        public void Draw(RectangleShape targetExtent, double drawingScreenWidth, double drawingScreenHeight)
        {
           // dynamic obj1 = SRg(new object[] { targetExtent, pointshape1.X, pointshape1.Y, drawingScreenWidth, drawingScreenHeight });
            ScreenPointF point1 = ExtentHelper.ToScreenCoordinate(targetExtent, pointshape1, (float)drawingScreenWidth, (float)drawingScreenHeight);
            ScreenPointF point2 = ExtentHelper.ToScreenCoordinate(targetExtent, centershape, (float)drawingScreenWidth, (float)drawingScreenHeight);
            ScreenPointF point3 = ExtentHelper.ToScreenCoordinate(targetExtent, pointshape2, (float)drawingScreenWidth, (float)drawingScreenHeight);

            Point pt = new Point();
            if (point1.X < point3.X)
            {
                pt.X = point1.X;
            }
            else
            {
                pt.X = point3.X;
            }
            if (point1.Y < point3.Y)
            {
                pt.Y = point1.Y;
            }
            else
            {
                pt.Y = point3.Y;
            }

            UpdateGisPathData(new Point(point1.X,point1.Y), new Point(point2.X, point2.Y), new Point(point3.X, point3.Y));
            Canvas.SetLeft(gisPath, pt.X);
            Canvas.SetTop(gisPath, pt.Y);
        }

        public void UpdateGisPathData(Point pt1,Point pt2,Point pt3)
        {
            PathGeometry pg = new PathGeometry();
            PathFigureCollection figures = new PathFigureCollection();

            PathFigure pathFigure = new PathFigure();
            pathFigure.StartPoint = pt1;
            PathSegmentCollection segmentCollection = new PathSegmentCollection();
            segmentCollection.Add(new BezierSegment()
            {
                Point1 = pt1,
                Point2 = pt2,
                Point3 = pt3
            });

            pathFigure.Segments = segmentCollection;
            figures.Add(pathFigure);

            pg.Figures = figures;
            this.gisPath.Data = pg;
        }
    }
}
