﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Wpf;

namespace ThinkGeoGis.GisCore
{
    [Serializable]
    public class PathOverylay : Overlay
    {
        [Obfuscation(Exclude = true)]
        private List<GisPath> paths;
        public List<GisPath> Paths
        {
            get
            {
                return paths;
            }
        }
        public PathOverylay() : this(new Collection<GisPath>())
        {
        }

        public PathOverylay(IEnumerable<GisPath> popups)
        {
            base.CanRefreshRegion = true;
            base.OverlayCanvas.SetValue(Panel.ZIndexProperty, 902);
            this.paths = new List<GisPath>();
            foreach (GisPath current in popups)
            {
                this.paths.Add(current);
            }
        }

        protected override RectangleShape GetBoundingBoxCore()
        {
            if (this.paths.Count > 0)
            {
                MultilineShape multipointShape = new MultilineShape();
                List<Vertex> vert = new List<Vertex>();

                foreach (GisPath current in this.paths)
                {
                    vert.Clear();
                    vert.Add(new Vertex(current.pointshape1));
                    vert.Add(new Vertex(current.pointshape2));
                    multipointShape.Lines.Add(new LineShape(vert));
                }
                return multipointShape.GetBoundingBox();
            }
            return base.GetBoundingBoxCore();
        }
        protected override void DrawCore(RectangleShape targetExtent, OverlayRefreshType overlayRefreshType)
        {
            Validators.CheckOverlayRefreshTypeIsValid(overlayRefreshType, "overlayRefreshType");
            if (overlayRefreshType == OverlayRefreshType.Pan)
            {
                IEnumerator enumerator = base.OverlayCanvas.Children.GetEnumerator();
                try
                {
                    while (enumerator.MoveNext())
                    {
                        var gisp = paths.Find(s => s.gisPath.GetHashCode() == enumerator.Current.GetHashCode());
                        if (gisp != null)
                            gisp.Draw(targetExtent, base.MapArguments.ActualWidth, base.MapArguments.ActualHeight);
                    }
                    return;
                }
                finally
                {
                    IDisposable disposable = enumerator as IDisposable;
                    if (disposable != null)
                    {
                        disposable.Dispose();
                    }
                }
            }
            base.OverlayCanvas.Children.Clear();
            foreach (GisPath current in this.paths)
            {
                if (!base.OverlayCanvas.Children.Contains(current.gisPath))
                    base.OverlayCanvas.Children.Add(current.gisPath);
                current.Draw(targetExtent, base.MapArguments.ActualWidth, base.MapArguments.ActualHeight);
            }
            StartAnimation();

            this.OnDrawn(new DrawnOverlayEventArgs(targetExtent));
        }

        public virtual void StartAnimation()
        {

        }
    }
}
