﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.Wpf;
using ThinkGeoGis.GisCore;
using ThinkGeoGis.Helper;

namespace ThinkGeoGis
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        private DataTable data;
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            data = ExcelHelper.Instance.ExcelToDataTable(dataFile.Text, "mapdata");
            MapControl.MapUnit = GeographyUnit.DecimalDegree;
            // We create a new Layer and pass the path to a Shapefile into its constructor. 
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer();
            worldLayer.RequireIndex = false;
            worldLayer.ShapePathFilename = @"..\..\Data\world_adm0.shp";
            AreaStyle areaStyle = new AreaStyle();
            areaStyle.FillSolidBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 233, 232, 214));
            areaStyle.OutlinePen = new GeoPen(GeoColor.FromArgb(255, 118, 138, 69), 1);
            areaStyle.OutlinePen.DashStyle = LineDashStyle.Solid;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = areaStyle;
            // This setting will apply from ZoonLevel01 to ZoomLevel20, that means we can see the world the same style with ZoomLevel01 all the time no matter how far we zoom out/in.
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            // Create a new Layer Overlay to hold the layer we just created
            LayerOverlay layerOverlay = new LayerOverlay();
            // Add a background Layer
            //layerOverlay.Layers.Add(new BackgroundLayer(new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean)));
            // Add the shapefile layer to the layer overlay
            layerOverlay.Layers.Add("MapLay", worldLayer);

            LayerOverlay lineOverlay = new LayerOverlay();
            InMemoryFeatureLayer linelay = new InMemoryFeatureLayer();
            linelay.DrawingQuality = DrawingQuality.HighQuality;
            linelay.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.GetRandomGeoColor(RandomColorType.All), 2));
            linelay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            lineOverlay.Layers.Add("LineLay", linelay);

            LayerOverlay pointOverlay = new LayerOverlay();
            InMemoryFeatureLayer pointlay = new InMemoryFeatureLayer();
            pointlay.DrawingQuality = DrawingQuality.HighQuality;
            pointlay.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.GetRandomGeoColor(RandomColorType.All)), 20);
            pointlay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            pointOverlay.Layers.Add("PointLay", pointlay);
            // We need to add the layerOverlay to map.


            LayerOverlay textOverlay = new LayerOverlay();
            InMemoryFeatureLayer textlay = new InMemoryFeatureLayer();
            
            textOverlay.Layers.Add("TextLay", textlay);

            MapControl.Overlays.Add("MapOverlay", layerOverlay);
            MapControl.Overlays.Add("LineOverlay", lineOverlay);
            MapControl.Overlays.Add("PointOverlay", pointOverlay);
            MapControl.Overlays.Add("TextOverlay", textOverlay);


            PathAnuimationOverylay pathOverylay = new PathAnuimationOverylay();
            MapControl.Overlays.Add("PathAnuimationOverylay", pathOverylay);
            // Set a proper extent for the Map.  
            MapControl.CurrentExtent = new RectangleShape(-180, 90, 180, -90);
            // We now need to call the Refresh() method of the Map control so that the Map can redraw based on the data that has been provided.
            MapControl.Refresh();
        }


        private void showDot_Click(object sender, RoutedEventArgs e)
        {
            var pointsFeature = GetDotPointShape();
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("PointLay");
            inMemoryLayer.InternalFeatures.Clear();
            foreach (var feature in pointsFeature)
            {
                inMemoryLayer.InternalFeatures.Add(feature);
            }
            MapControl.Overlays["PointOverlay"].Refresh();
        }


        private void showLine_Click(object sender, RoutedEventArgs e)
        {
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("LineLay");
            inMemoryLayer.InternalFeatures.Clear();
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() == "" || data.Rows[i][3].ToString() == "" || data.Rows[i][6].ToString() == "" || data.Rows[i][7].ToString() == "")
                    continue;

                LineShapeHelper lineshapeHelper = new LineShapeHelper();
                lineshapeHelper.DestinationLocation = new PointShape(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString()));
                lineshapeHelper.OriginLocation = new PointShape(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString()));
                var mutlineshape = lineshapeHelper.GetArc(2);
                foreach (var lineshape in mutlineshape.Lines)
                {
                    inMemoryLayer.InternalFeatures.Add(new Feature(lineshape));
                }
            }
            MapControl.Overlays["LineOverlay"].Refresh();
        }

        private GeoCollection<Feature> GetDotPointShape()
        {
            if (data == null)
                return null;
            GeoCollection<Feature> pointShpeList = new GeoCollection<Feature>();

            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() != "" && data.Rows[i][3].ToString() != "")
                    pointShpeList.Add(new Feature(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString())));
                if (data.Rows[i][6].ToString() != "" && data.Rows[i][7].ToString() != "")
                    pointShpeList.Add(new Feature(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString())));
            }
            return pointShpeList;
        }

        private void showText_Click(object sender, RoutedEventArgs e)
        {

            InMemoryFeatureLayer layer = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("TextLay");
            layer.Columns.Add(new FeatureSourceColumn("label"));
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() != "" && data.Rows[i][3].ToString() != "")
                {
                    Feature feature1 = new Feature(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString()));
                    feature1.ColumnValues.Add("label", data.Rows[i][1].ToString());
                    layer.InternalFeatures.Add(feature1);
                }
                if (data.Rows[i][6].ToString() != "" && data.Rows[i][7].ToString() != "")
                {
                    Feature feature1 = new Feature(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString()));
                    feature1.ColumnValues.Add("label", data.Rows[i][5].ToString());
                    layer.InternalFeatures.Add(feature1);
                }
            }
            layer.DrawingQuality = DrawingQuality.HighQuality;
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("label");
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            MapControl.Overlays["TextOverlay"].Refresh();
        }

        private void showPathLine_Click(object sender, RoutedEventArgs e)
        {
            var textOverlay = MapControl.Overlays["PathAnuimationOverylay"] as PathAnuimationOverylay;
            textOverlay.OverlayCanvas.Children.Clear();
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() == "" || data.Rows[i][3].ToString() == "" || data.Rows[i][6].ToString() == "" || data.Rows[i][7].ToString() == "")
                    continue;
                //if (data.Rows[i][6].ToString() != "- 77.020857" & data.Rows[i][7].ToString() != "38.899608")
                //    continue;
                GisPath gispath = new GisPath(new PointShape(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString())),new PointShape(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString())));
                textOverlay.Paths.Add(gispath);
            }
            textOverlay.Refresh();
        }

        private void showPathAnimation_Click(object sender, RoutedEventArgs e)
        {
            var textOverlay = MapControl.Overlays["PathAnuimationOverylay"] as PathAnuimationOverylay;
            textOverlay.Refresh();
            textOverlay.StartAnimation();
        }


        private void MapControl_MapClick(object sender, MapClickWpfMapEventArgs e)
        {
            screentext.Text = string.Format("屏幕坐标{0},{1}", e.ScreenX.ToString(), e.ScreenY.ToString());
            worldtext.Text = string.Format("世界坐标{0},{1}", e.WorldX.ToString(), e.WorldY.ToString());
        }
    }
}
